summaryrefslogtreecommitdiffstats
path: root/util/intelp2m
diff options
context:
space:
mode:
authorMatt DeVillier <matt.devillier@gmail.com>2020-08-09 14:13:56 -0500
committerPatrick Georgi <pgeorgi@google.com>2020-08-18 08:49:27 +0000
commit5eeead2d73d2daa2361d0272db41e85e1de79a6c (patch)
treeaa6663f6db6996a79cbbc16d0cd97070d7fa3553 /util/intelp2m
parentceb409a2a69075c060736845dc4a444bcb6c212e (diff)
downloadcoreboot-5eeead2d73d2daa2361d0272db41e85e1de79a6c.tar.gz
coreboot-5eeead2d73d2daa2361d0272db41e85e1de79a6c.tar.bz2
coreboot-5eeead2d73d2daa2361d0272db41e85e1de79a6c.zip
util/intelp2m: Add support for Cannonlake-LP SoCs
Add support for Cannonlake-LP SoCs (Whiskeylake-U, Coffeelake-U, Cometlake-U) as a separate parsing profile, copying the existing 'Sunrise' profile and adjusting for differences in reset mapping and GPIO macro generation Test: convert inteltool GPIO log dump into coreboot macros for an out-of-tree CML-U board. Change-Id: I86296697ee892af7aa0818fb608b6d68fad2f307 Signed-off-by: Matt DeVillier <matt.devillier@gmail.com> Reviewed-on: https://review.coreboot.org/c/coreboot/+/44457 Tested-by: build bot (Jenkins) <no-reply@coreboot.org> Reviewed-by: Maxim Polyakov <max.senia.poliak@gmail.com>
Diffstat (limited to 'util/intelp2m')
-rw-r--r--util/intelp2m/config/config.go7
-rw-r--r--util/intelp2m/description.md3
-rw-r--r--util/intelp2m/main.go3
-rw-r--r--util/intelp2m/parser/parser.go6
-rw-r--r--util/intelp2m/platforms/cnl/macro.go215
-rw-r--r--util/intelp2m/platforms/cnl/template.go33
6 files changed, 263 insertions, 4 deletions
diff --git a/util/intelp2m/config/config.go b/util/intelp2m/config/config.go
index 9f0b75772c55..724de8caa62e 100644
--- a/util/intelp2m/config/config.go
+++ b/util/intelp2m/config/config.go
@@ -27,6 +27,7 @@ const (
SunriseType uint8 = 0
LewisburgType uint8 = 1
ApolloType uint8 = 2
+ CannonType uint8 = 3
)
var key uint8 = SunriseType
@@ -34,7 +35,8 @@ var key uint8 = SunriseType
var platform = map[string]uint8{
"snr": SunriseType,
"lbg": LewisburgType,
- "apl": ApolloType}
+ "apl": ApolloType,
+ "cnl": CannonType}
func PlatformSet(name string) int {
if platformType, valid := platform[name]; valid {
key = platformType
@@ -57,6 +59,9 @@ func IsPlatformSunrise() bool {
func IsPlatformLewisburg() bool {
return IsPlatform(LewisburgType)
}
+func IsPlatformCannonLake() bool {
+ return IsPlatform(CannonType)
+}
var InputRegDumpFile *os.File = nil
var OutputGenFile *os.File = nil
diff --git a/util/intelp2m/description.md b/util/intelp2m/description.md
index 3008c046abed..add9c2cf88af 100644
--- a/util/intelp2m/description.md
+++ b/util/intelp2m/description.md
@@ -37,6 +37,7 @@ platform type is set using the -p option (Sunrise by default):
snr - Sunrise PCH with Skylake/Kaby Lake CPU
lbg - Lewisburg PCH with Xeon SP CPU
apl - Apollo Lake SoC
+ cnl - CannonLake-LP or Whiskeylake/Coffelake/Cometlake-U SoC
(default "snr")
(shell)$ ./intelp2m -p <platform> -file path/to/inteltool.log
@@ -198,4 +199,4 @@ PAD_CFG_NF_IOSSTATE_IOSTERM(GPIO_39, UP_20K, DEEP, NF1, TxLASTRxE, DISPUPD),
```
### Supports Chipsets
- Sunrise PCH, Lewisburg PCH, Apollo Lake SoC
+ Sunrise PCH, Lewisburg PCH, Apollo Lake SoC, CannonLake-LP SoCs
diff --git a/util/intelp2m/main.go b/util/intelp2m/main.go
index 6c6dc34369a5..8527c5458689 100644
--- a/util/intelp2m/main.go
+++ b/util/intelp2m/main.go
@@ -81,7 +81,8 @@ func main() {
platform := flag.String("p", "snr", "set platform:\n"+
"\tsnr - Sunrise PCH or Skylake/Kaby Lake SoC\n"+
"\tlbg - Lewisburg PCH with Xeon SP\n"+
- "\tapl - Apollo Lake SoC\n")
+ "\tapl - Apollo Lake SoC\n"+
+ "\tcnl - CannonLake-LP or Whiskeylake/Coffelake/Cometlake-U SoC\n")
filedstyle := flag.String("fld", "none", "set fileds macros style:\n"+
"\tcb - use coreboot style for bit fields macros\n"+
diff --git a/util/intelp2m/platforms/cnl/macro.go b/util/intelp2m/platforms/cnl/macro.go
new file mode 100644
index 000000000000..c3bdfc88b115
--- /dev/null
+++ b/util/intelp2m/platforms/cnl/macro.go
@@ -0,0 +1,215 @@
+package cnl
+
+import "strings"
+import "fmt"
+
+// Local packages
+import "../common"
+import "../../config"
+import "../../fields"
+import "../snr"
+
+const (
+ PAD_CFG_DW0_RO_FIELDS = (0x1 << 27) | (0x1 << 24) | (0x3 << 21) | (0xf << 16) | 0xfc
+ PAD_CFG_DW1_RO_FIELDS = 0xfdffc3ff
+)
+
+const (
+ PAD_CFG_DW0 = common.PAD_CFG_DW0
+ PAD_CFG_DW1 = common.PAD_CFG_DW1
+ MAX_DW_NUM = common.MAX_DW_NUM
+)
+
+type InheritanceMacro interface {
+ GpoMacroAdd()
+ NativeFunctionMacroAdd()
+ NoConnMacroAdd()
+}
+
+type PlatformSpecific struct {
+ InheritanceMacro
+ InheritanceTemplate
+}
+
+// RemmapRstSrc - remmap Pad Reset Source Config
+func (PlatformSpecific) RemmapRstSrc() {
+ macro := common.GetMacro()
+ if config.TemplateGet() != config.TempInteltool {
+ // Use reset source remapping only if the input file is inteltool.log dump
+ return
+ }
+ if strings.Contains(macro.PadIdGet(), "GPP_A") ||
+ strings.Contains(macro.PadIdGet(), "GPP_B") ||
+ strings.Contains(macro.PadIdGet(), "GPP_G") {
+ // See reset map for the Cannonlake Groups the Community 0:
+ // https://github.com/coreboot/coreboot/blob/master/src/soc/intel/cannonlake/gpio.c#L14
+ // remmap is not required because it is the same as common.
+ return
+ }
+
+ dw0 := macro.Register(PAD_CFG_DW0)
+ var remapping = map[uint8]uint32{
+ 0: common.RST_RSMRST << common.PadRstCfgShift,
+ 1: common.RST_DEEP << common.PadRstCfgShift,
+ 2: common.RST_PLTRST << common.PadRstCfgShift,
+ }
+ resetsrc, valid := remapping[dw0.GetResetConfig()]
+ if valid {
+ // dw0.SetResetConfig(resetsrc)
+ ResetConfigFieldVal := (dw0.ValueGet() & 0x3fffffff) | remapping[dw0.GetResetConfig()]
+ dw0.ValueSet(ResetConfigFieldVal)
+ } else {
+ fmt.Println("Invalid Pad Reset Config [ 0x", resetsrc ," ] for ", macro.PadIdGet())
+ }
+ dw0.CntrMaskFieldsClear(common.PadRstCfgMask)
+}
+
+// Adds The Pad Termination (TERM) parameter from PAD_CFG_DW1 to the macro
+// as a new argument
+func (PlatformSpecific) Pull() {
+ macro := common.GetMacro()
+ dw1 := macro.Register(PAD_CFG_DW1)
+ var pull = map[uint8]string{
+ 0x0: "NONE",
+ 0x2: "DN_5K",
+ 0x4: "DN_20K",
+ 0x9: "UP_1K",
+ 0xa: "UP_5K",
+ 0xb: "UP_2K",
+ 0xc: "UP_20K",
+ 0xd: "UP_667",
+ 0xf: "NATIVE",
+ }
+ str, valid := pull[dw1.GetTermination()]
+ if !valid {
+ str = "INVALID"
+ fmt.Println("Error",
+ macro.PadIdGet(),
+ " invalid TERM value = ",
+ int(dw1.GetTermination()))
+ }
+ macro.Separator().Add(str)
+}
+
+// Generate macro to cause peripheral IRQ when configured in GPIO input mode
+func ioApicRoute() bool {
+ macro := common.GetMacro()
+ dw0 := macro.Register(PAD_CFG_DW0)
+ if dw0.GetGPIOInputRouteIOxAPIC() == 0 {
+ return false
+ }
+
+ macro.Add("_APIC")
+ // PAD_CFG_GPI_APIC(pad, pull, rst, trig, inv)
+ macro.Add("(").Id().Pull().Rstsrc().Trig().Invert().Add("),")
+ return true
+}
+
+// Generate macro to cause NMI when configured in GPIO input mode
+func nmiRoute() bool {
+ macro := common.GetMacro()
+ if macro.Register(PAD_CFG_DW0).GetGPIOInputRouteNMI() == 0 {
+ return false
+ }
+ // PAD_CFG_GPI_NMI(GPIO_24, UP_20K, DEEP, LEVEL, INVERT),
+ macro.Add("_NMI").Add("(").Id().Pull().Rstsrc().Trig().Invert().Add("),")
+ return true
+}
+
+// Generate macro to cause SCI when configured in GPIO input mode
+func sciRoute() bool {
+ macro := common.GetMacro()
+ dw0 := macro.Register(PAD_CFG_DW0)
+ if dw0.GetGPIOInputRouteSCI() == 0 {
+ return false
+ }
+ // PAD_CFG_GPI_SCI(pad, pull, rst, trig, inv)
+ macro.Add("_SCI").Add("(").Id().Pull().Rstsrc().Trig().Invert().Add("),")
+ return true
+}
+
+// Generate macro to cause SMI when configured in GPIO input mode
+func smiRoute() bool {
+ macro := common.GetMacro()
+ dw0 := macro.Register(PAD_CFG_DW0)
+ if dw0.GetGPIOInputRouteSMI() == 0 {
+ return false
+ }
+ // PAD_CFG_GPI_SMI(pad, pull, rst, trig, inv)
+ macro.Add("_SMI").Add("(").Id().Pull().Rstsrc().Trig().Invert().Add("),")
+ return true
+}
+
+// Adds PAD_CFG_GPI macro with arguments
+func (PlatformSpecific) GpiMacroAdd() {
+ macro := common.GetMacro()
+ var ids []string
+ macro.Set("PAD_CFG_GPI")
+ for routeid, isRoute := range map[string]func() (bool) {
+ "IOAPIC": ioApicRoute,
+ "SCI": sciRoute,
+ "SMI": smiRoute,
+ "NMI": nmiRoute,
+ } {
+ if isRoute() {
+ ids = append(ids, routeid)
+ }
+ }
+
+ switch argc := len(ids); argc {
+ case 0:
+ // e.g. PAD_CFG_GPI_TRIG_OWN(pad, pull, rst, trig, own)
+ macro.Add("_TRIG_OWN").Add("(").Id().Pull().Rstsrc().Trig().Own().Add("),")
+ case 1:
+ // GPI with IRQ route
+ if config.AreFieldsIgnored() {
+ // Set Host Software Ownership to ACPI mode
+ macro.SetPadOwnership(common.PAD_OWN_ACPI)
+ }
+
+ case 2:
+ // PAD_CFG_GPI_DUAL_ROUTE(pad, pull, rst, trig, inv, route1, route2)
+ macro.Set("PAD_CFG_GPI_DUAL_ROUTE(").Id().Pull().Rstsrc().Trig().Invert()
+ macro.Add(", " + ids[0] + ", " + ids[1] + "),")
+ if config.AreFieldsIgnored() {
+ // Set Host Software Ownership to ACPI mode
+ macro.SetPadOwnership(common.PAD_OWN_ACPI)
+ }
+ default:
+ // Clear the control mask so that the check fails and "Advanced" macro is
+ // generated
+ macro.Register(PAD_CFG_DW0).CntrMaskFieldsClear(common.AllFields)
+ }
+}
+
+// Adds PAD_CFG_GPO macro with arguments
+func (platform PlatformSpecific) GpoMacroAdd() {
+ platform.InheritanceMacro.GpoMacroAdd()
+}
+
+// Adds PAD_CFG_NF macro with arguments
+func (platform PlatformSpecific) NativeFunctionMacroAdd() {
+ platform.InheritanceMacro.NativeFunctionMacroAdd()
+}
+
+// Adds PAD_NC macro
+func (platform PlatformSpecific) NoConnMacroAdd() {
+ platform.InheritanceMacro.NoConnMacroAdd()
+}
+
+// GenMacro - generate pad macro
+// dw0 : DW0 config register value
+// dw1 : DW1 config register value
+// return: string of macro
+// error
+func (PlatformSpecific) GenMacro(id string, dw0 uint32, dw1 uint32, ownership uint8) string {
+ macro := common.GetInstanceMacro(PlatformSpecific{InheritanceMacro : snr.PlatformSpecific{}},
+ fields.InterfaceGet())
+ macro.Clear()
+ macro.Register(PAD_CFG_DW0).CntrMaskFieldsClear(common.AllFields)
+ macro.Register(PAD_CFG_DW0).CntrMaskFieldsClear(common.AllFields)
+ macro.PadIdSet(id).SetPadOwnership(ownership)
+ macro.Register(PAD_CFG_DW0).ValueSet(dw0).ReadOnlyFieldsSet(PAD_CFG_DW0_RO_FIELDS)
+ macro.Register(PAD_CFG_DW1).ValueSet(dw1).ReadOnlyFieldsSet(PAD_CFG_DW1_RO_FIELDS)
+ return macro.Generate()
+}
diff --git a/util/intelp2m/platforms/cnl/template.go b/util/intelp2m/platforms/cnl/template.go
new file mode 100644
index 000000000000..f1a17413bce7
--- /dev/null
+++ b/util/intelp2m/platforms/cnl/template.go
@@ -0,0 +1,33 @@
+package cnl
+
+import "strings"
+
+type InheritanceTemplate interface {
+
+ KeywordCheck(line string) bool
+}
+
+// GroupNameExtract - This function extracts the group ID, if it exists in a row
+// line : string from the configuration file
+// return
+// bool : true if the string contains a group identifier
+// string : group identifier
+func (PlatformSpecific) GroupNameExtract(line string) (bool, string) {
+ for _, groupKeyword := range []string{
+ "GPP_A", "GPP_B", "GPP_G",
+ "GPP_D", "GPP_F", "GPP_H",
+ "GPD", "GPP_C", "GPP_E",
+ } {
+ if strings.Contains(line, groupKeyword) {
+ return true, groupKeyword
+ }
+ }
+ return false, ""
+}
+
+// KeywordCheck - This function is used to filter parsed lines of the configuration file and
+// returns true if the keyword is contained in the line.
+// line : string from the configuration file
+func (platform PlatformSpecific) KeywordCheck(line string) bool {
+ return platform.InheritanceTemplate.KeywordCheck(line)
+}
diff --git a/util/intelp2m/parser/parser.go b/util/intelp2m/parser/parser.go
index f002cc906409..8a58ab78a5f2 100644
--- a/util/intelp2m/parser/parser.go
+++ b/util/intelp2m/parser/parser.go
@@ -10,6 +10,7 @@ import (
import "../platforms/snr"
import "../platforms/lbg"
import "../platforms/apl"
+import "../platforms/cnl"
import "../config"
// PlatformSpecific - platform-specific interface
@@ -141,6 +142,9 @@ func (parser *ParserData) PlatformSpecificInterfaceSet() {
InheritanceTemplate : snr.PlatformSpecific{},
},
config.ApolloType : apl.PlatformSpecific{},
+ config.CannonType : cnl.PlatformSpecific{
+ InheritanceTemplate : snr.PlatformSpecific{},
+ },
}
parser.platform = platform[config.PlatformGet()]
}
@@ -198,7 +202,7 @@ func (parser *ParserData) padOwnershipExtract() bool {
// padConfigurationExtract - reads GPIO configuration registers and returns true if the
// information from the inteltool log was successfully parsed.
func (parser *ParserData) padConfigurationExtract() bool {
// Only for Sunrise PCH and only for inteltool.log file template
if config.TemplateGet() != config.TempInteltool || config.IsPlatformApollo() {
return false
}
@@ -198,7 +202,7 @@ func (parser *ParserData) padOwnershipExtract() bool {
// information from the inteltool log was successfully parsed.
func (parser *ParserData) padConfigurationExtract() bool {
- // Only for Sunrise PCH and only for inteltool.log file template
+ // Only for Sunrise or CannonLake, and only for inteltool.log file template
if config.TemplateGet() != config.TempInteltool || config.IsPlatformApollo() {
return false